/*
 * ATT_PE4302_ATMEGA328P
 *
 * main.c
 *
 * Created:		Вс 11.02.24 21:42:50
 * Author:		R9OFG.RU
 * Release:		1.0
 * Modified:	12-02-2024
 */ 

/* инклюды */

#include "main.h"

/* переменные */

volatile int8_t new_state	= 0;	//для энкодера
volatile int8_t old_state	= 0;	//-
volatile int8_t up_state	= 0;	//-
volatile int8_t down_state	= 0;	//-
volatile int8_t rotate		= 0;	//-

volatile int8_t att_data	= 0;	//биты кправления аттенюатором PE4302
volatile int8_t corr		= 0;	//значение калибровки

volatile uint8_t but_delay	= 0;	//счетчик нажатой кнопки
volatile uint8_t flag_test	= 0;	//флаг запущенного теста
volatile uint8_t flag_calib	= 0;	//флаг запущенной калибровки

char strValueAtt[10]		= "";	//строка для вывода значений затухания аттенюатора

uint8_t test_rotate			= 0;	//направление для теста

/* внешние переменные */

extern uint8_t	sign_number;		//Float_transform
extern int		integer_number;		//Float_transform
extern uint32_t fractional_number;	//Float_transform

/* функции */

/* функция инициализация 16-битного таймера TC1 */
void TIMER1_init()
{
	TCCR1A = 0x00;			//normal режим и счетчик
	TCCR1B |= (1<<CS11);    //тактировать с коэффициентом деления 8
	TCNT1H = 0x00;          //обнуляем счетный регистр старший байт
	TCNT1L = 0x00;          //обнуляем счетный регистр младший байт
	OCR1AH = 0x03;          //настраиваем регистр сравнения старший байт
	OCR1AL = 0xE8;          //настраиваем регистр сравнения младший байт
}

/* функция вывода информации на дисплей */
void DISPLAY_output()
{
	//att_data делим на 2, чтобы выводимый результат привести к значению в dB
	Float_transform((float)att_data / 2.0f + (float)corr / 10, 1, &sign_number, &integer_number, &fractional_number);
	if (!sign_number)
	{
		sprintf((char*)&strValueAtt, "-%d.%.01ld\n", integer_number, fractional_number);
	}
	
	OLED_clear();
	
	if(flag_test) {PlotChar('T',0,0);} //если тест, выводим метку на дисплей
	if(flag_calib) {PlotChar('C',0,0);} //если калибровка, выводим метку на дисплей
	
	uint8_t y;
	if(((float)att_data / 2.0f) < 10)
	{
		y = 35; //координата Х начала вывода строки
	}
	else
	{
		y = 30; //координата Х начала вывода строки
	}
	for (int i=0;strValueAtt[i]!='\n';i++)
	{
		PlotChar(strValueAtt[i],0,y);
		y += 13;
	}
	PlotText(PSTR("dB"), 2, 53);
}

/* функция управления аттенюатором PE4302 */
void PE4302_change_att(uint8_t direction)
{
	if(direction == 0)	//увеличиваем затухание
	{
		att_data++;
		
		//если достигли предела выходим из функции
		if(att_data > 63)
		{
			att_data = 63;
			test_rotate = 1;
			return;
		}
	}
	
	if(direction == 1)	//уменьшаем затухание
	{
		att_data--;
		//если достигли предела выходим из функции
		if(att_data < 0)
		{
			att_data = 0;
			test_rotate = 0;
			return;
		}
	}

	//проверяем состояние управляющих битов и выставляем соответствующий логический уровень на пинах для управления PE4302 V1...V6
	if(att_data & (1<<0)) {PE_V1_set;} else {PE_V1_reset;}
	if(att_data & (1<<1)) {PE_V2_set;} else {PE_V2_reset;}
	if(att_data & (1<<2)) {PE_V3_set;} else {PE_V3_reset;}
	if(att_data & (1<<3)) {PE_V4_set;} else {PE_V4_reset;}
	if(att_data & (1<<4)) {PE_V5_set;} else {PE_V5_reset;}
	if(att_data & (1<<5)) {PE_V6_set;} else {PE_V6_reset;}
		
	//выводим значение затухания на дисплей
	DISPLAY_output();
}

/* функция циклического прогона аттенюатора */
void TEST_att()
{
	while(flag_test != 0)
	{
		//если кнопка энкодера нажата, прерываем цикл
		if((1 << BUT) & PINB)
		{
			flag_test = 0;
		}
		
		if(test_rotate == 0)
		{
			PE4302_change_att(0);
		}
		
		if(test_rotate == 1)
		{
			PE4302_change_att(1);
		}
		
		_delay_ms(TEST_ATT_DELAY);
	}
	
	//сбрасываем затухание аттенюатора
	att_data = 0;
	PE4302_change_att(2);
}

/* функция вывода заставки */
void Splash()
{
	//заставка
	OLED_clear();
	PlotText(PSTR("ATT"), 0, 48);
	PlotText(PSTR("PE-4302"), 2, 22);
	_delay_ms(1500);
	OLED_clear();	
	PlotText(PSTR("1.0"), 0, 48);		//релиз!!!
	PlotText(PSTR("R9OFG.RU"), 2, 17);
	_delay_ms(1500);
}

/* прерывания */

ISR(TIMER1_COMPA_vect)			//обработчик таймера 1
{
	
	TCNT1H = 0x00; //обнуляем счетчик таймера
	TCNT1L = 0x00; //-
	
	new_state = PINB & 0x06;	//маска пинов PB1 и PB2 для энкодера - 0b00000110
	
	if(new_state != old_state)
	{
		switch(old_state)
		{
			case 2:
			{
				if(new_state == 3) up_state++;
				if(new_state == 0) down_state++;
				break;
			}
			
			case 0:
			{
				if(new_state == 2) up_state++;
				if(new_state == 1) down_state++;
				break;
			}
			case 1:
			{
				if(new_state == 0) up_state++;
				if(new_state == 3) down_state++;
				break;
			}
			case 3:
			{
				if(new_state == 1) up_state++;
				if(new_state == 2) down_state++;
				break;
			}
		}
		old_state=new_state;
	}
	
	//если кнопка энкодера нажата
	if((1 << BUT) & PINB)
	{
		_delay_ms(500);
		if((1 << BUT) & PINB)
		{
			if(flag_calib == 1)
			{
				//если включен режим калибровки, сохраняем значение калибровки в EEPROM, сбрасываем аттенюатор и включаем рабочий режим
				flag_calib = 0;
				eeprom_write_byte((uint8_t*)1, (uint8_t)corr);
				att_data = 0;
				PE4302_change_att(2);
				return;
			}
				
			//инкремент счетчика нажатой кнопки энкодера
			but_delay++;
			if(but_delay > 254) {but_delay = 0;}
		}
	}
	else
	{
		if((but_delay > 1)&&(but_delay < 3)&&(flag_test == 0))
		{
			//поднимаем флаг теста
			flag_test = 1;
			//запускаем тест аттенюатора
			TEST_att();
		}
		
		if(but_delay > 5)
		{
			//поднимаем флаг калибровки
			flag_calib = 1;
			//сбрасываем затухание аттенюатора
			att_data = 0;
			PE4302_change_att(2);	
		}
				
		//сбрасываем счетчик нажатой кнопки энкодера
		but_delay = 0;	
	}
}

/* секция main */

int main(void)
{
    /* настройка портов */
	
	DDRB	= 0x00;
	PORTB	= 0x06;			/*
							  пины PB1 и PB2 лог. ед, крайние выводы на энкодере,
							  средний вывод подключется к GND,
							  если при напралении вращения по часовой стрелке затухание увеличивается,
							  поменять выводы местами
							 */
	
	DDRD	= 0xFF;			//к пинам PD0...PD5 подключены пины PE4302 V1...V6
	PORTD	= 0x00;			//присваиваем всем пинам порта D логический ноль 0b00000000
	
	/* инициализация модулей */
	
	I2C_init();				//инициализация шины i2c
	OLED_init();			//инициализация дисплея
	TIMER1_init();			//инициализация таймера 1
	
	/* чтение калибровки из EEPROM */
	
	corr = eeprom_read_byte((uint8_t*)1);
	if((corr < 0)||(corr > 20))
	{
		//значение не корректное, сохраняем по умолчанию
		corr = 0;
		eeprom_write_byte((uint8_t*)1, (uint8_t)corr);
	}	
	
	Splash();				//выводим заставку
		
	TIMSK1 |= (1<<OCIE1A);	//разрешаем прерывание таймера по совпадению с OCR1A
	
	sei();					//разрешаем глобальные прерывания
	
	att_data = 0;			//сброс аттенюатора
	PE4302_change_att(2);
	
	/* основной цикл */
	
	while(1) 
    {
		if(up_state >= ENCODER_STEP)
		{
			rotate++;
			up_state = 0;
			if(flag_calib != 0)
			{
				//увеличиваем значение корректировки, но не более 2
				corr += 1;
				if(corr > 20)
				{
					corr = 20;
				}
				else
				{
					DISPLAY_output();
				}
				_delay_ms(10);
			}
			else
			{
				//увеличиваем затухание аттенюатора
				PE4302_change_att(0);
			}
		}
		if (down_state >= ENCODER_STEP)
		{
			rotate--;
			down_state = 0;
			if(flag_calib != 0)
			{
				//уменьшаем значение корректировки, но не меньше 0
				corr -= 1;
				if(corr < 0)
				{
					corr = 0;
				}
				else
				{
					DISPLAY_output();
				}
				_delay_ms(10);
			}
			else
			{
				//уменьшаем затухания аттенюатора
				PE4302_change_att(1);
			}
		}
    }
}

/* конец кода */

